@@ -1,7 +1,7 @@
 /* 
  *  Squeezelite - lightweight headless squeezebox emulator
  *
- *  (c) Adrian Smith 2012-2014, triode1@btinternet.com
+ *  (c) Adrian Smith 2012-2015, triode1@btinternet.com
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,7 +49,9 @@ frames_t _output_frames(frames_t avail) {
 	
 	s32_t gainL = output.current_replay_gain ? gain(output.gainL, output.current_replay_gain) : output.gainL;
 	s32_t gainR = output.current_replay_gain ? gain(output.gainR, output.current_replay_gain) : output.gainR;
-	
+
+	if (output.invert) { gainL = -gainL; gainR = -gainR; }
+
 	frames = _buf_used(outputbuf) / BYTES_PER_FRAME;
 	silence = false;
 
@@ -220,6 +222,7 @@ frames_t _output_frames(frames_t avail) {
 						fade_gain = to_gain((float)cur_f / (float)dur_f);
 						gainL = gain(gainL, fade_gain);
 						gainR = gain(gainR, fade_gain);
+						if (output.invert) { gainL = -gainL; gainR = -gainR; }
 					}
 					if (output.fade_dir == FADE_CROSS) {
 						// cross fade requires special treatment - performed later based on these values
@@ -235,6 +238,7 @@ frames_t _output_frames(frames_t avail) {
 							}
 							gainL = output.gainL;
 							gainR = output.gainR;
+							if (output.invert) { gainL = -gainL; gainR = -gainR; }
 							cross_ptr = (s32_t *)(output.fade_end + cur_f * BYTES_PER_FRAME);
 						} else {
 							LOG_INFO("unable to continue crossfade - too few samples");
@@ -333,7 +337,7 @@ void _checkfade(bool start) {
 	}
 }
 
-void output_init_common(log_level level, const char *device, unsigned output_buf_size, unsigned rates[]) {
+void output_init_common(log_level level, const char *device, unsigned output_buf_size, unsigned rates[], unsigned idle) {
 	unsigned i;
 
 	loglevel = level;
@@ -363,10 +367,14 @@ void output_init_common(log_level level, const char *device, unsigned output_buf
 		dop_silence_frames((u32_t *)silencebuf_dop, MAX_SILENCE_FRAMES);
 	)
 
-	output.state = OUTPUT_STOPPED;
+	LOG_DEBUG("idle timeout: %u", idle);
+
+	output.state = idle ? OUTPUT_OFF: OUTPUT_STOPPED;
 	output.device = device;
 	output.fade = FADE_INACTIVE;
+	output.invert = false;
 	output.error_opening = false;
+	output.idle_to = (u32_t) idle;
 
 	if (!rates[0]) {
 		if (!test_open(output.device, output.supported_rates)) {
@@ -387,7 +395,7 @@ void output_init_common(log_level level, const char *device, unsigned output_buf
 		}
 	}
 	if (!output.default_sample_rate) {
-		output.default_sample_rate = rates[0];
+		output.default_sample_rate = output.supported_rates[0];
 	}
 	
 	output.current_sample_rate = output.default_sample_rate;
