@@ -1,7 +1,7 @@
 /* 
  *  Squeezelite - lightweight headless squeezebox emulator
  *
- *  (c) Adrian Smith 2012-2014, triode1@btinternet.com
+ *  (c) Adrian Smith 2012-2015, triode1@btinternet.com
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@ extern u8_t *silencebuf_dop;
 #endif
 
 void list_devices(void) {
- 	PaError err;
+	PaError err;
 	int i;
 
 	if ((err = Pa_Initialize()) != paNoError) {
@@ -67,11 +67,19 @@ void list_devices(void) {
 	}
 	printf("\n");
 	
- 	if ((err = Pa_Terminate()) != paNoError) {
+	if ((err = Pa_Terminate()) != paNoError) {
 		LOG_WARN("error closing port audio: %s", Pa_GetErrorText(err));
 	}
 }
 
+void set_volume(unsigned left, unsigned right) {
+	LOG_DEBUG("setting internal gain left: %u right: %u", left, right);
+	LOCK;
+	output.gainL = left;
+	output.gainR = right;
+	UNLOCK;
+}
+
 static int pa_device_id(const char *device) {
 	int len = strlen(device);
 	int i;
@@ -306,7 +314,7 @@ static int _write_frames(frames_t out_frames, bool silence, s32_t gainL, s32_t g
 
 		IF_DSD(
 			if (output.dop) {
-				update_dop_marker((u32_t *) outputbuf->readp, out_frames);
+				update_dop((u32_t *) outputbuf->readp, out_frames, output.invert);
 			}
 		)
 
@@ -319,7 +327,7 @@ static int _write_frames(frames_t out_frames, bool silence, s32_t gainL, s32_t g
 		IF_DSD(
 			if (output.dop) {
 				buf = silencebuf_dop;
-				update_dop_marker((u32_t *) buf, out_frames);
+				update_dop((u32_t *) buf, out_frames, false); // don't invert silence
 			}
 		)
 
@@ -377,7 +385,8 @@ static int pa_callback(const void *pa_input, void *pa_output, unsigned long pa_f
 	return ret;
 }
 
-void output_init_pa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[], unsigned rate_delay) {
+void output_init_pa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[], unsigned rate_delay,
+					unsigned idle) {
 	PaError err;
 	unsigned latency = 0;
 	int osx_playnice = -1;
@@ -404,12 +413,12 @@ void output_init_pa(log_level level, const char *device, unsigned output_buf_siz
 
 	LOG_INFO("requested latency: %u", output.latency);
 
- 	if ((err = Pa_Initialize()) != paNoError) {
+	if ((err = Pa_Initialize()) != paNoError) {
 		LOG_WARN("error initialising port audio: %s", Pa_GetErrorText(err));
 		exit(0);
 	}
 
-	output_init_common(level, device, output_buf_size, rates);
+	output_init_common(level, device, output_buf_size, rates, idle);
 
 	LOCK;
 
@@ -434,7 +443,7 @@ void output_close_pa(void) {
 		}
 	}
 
- 	if ((err = Pa_Terminate()) != paNoError) {
+	if ((err = Pa_Terminate()) != paNoError) {
 		LOG_WARN("error closing port audio: %s", Pa_GetErrorText(err));
 	}
 
