@@ -1,7 +1,7 @@
 /* 
  *  Squeezelite - lightweight headless squeezebox emulator
  *
- *  (c) Adrian Smith 2012-2014, triode1@btinternet.com
+ *  (c) Adrian Smith 2012-2015, triode1@btinternet.com
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@ static int _stdout_write_frames(frames_t out_frames, bool silence, s32_t gainL,
 			   if (silence) {
 				   obuf = silencebuf_dop;
 			   }
-			   update_dop_marker((u32_t *)obuf, out_frames);
+			   update_dop((u32_t *)obuf, out_frames, output.invert && !silence);
 		   }
 	)
 
@@ -150,7 +150,7 @@ void output_init_stdout(log_level level, unsigned output_buf_size, char *params,
 		rates[0] = 44100;
 	}
 
-	output_init_common(level, "-", output_buf_size, rates);
+	output_init_common(level, "-", output_buf_size, rates, 0);
 
 #if LINUX || OSX || FREEBSD
 	pthread_attr_t attr;
