@@ -1,7 +1,7 @@
 /* 
  *  Squeezelite - lightweight headless squeezebox emulator
  *
- *  (c) Adrian Smith 2012-2014, triode1@btinternet.com
+ *  (c) Adrian Smith 2012-2015, triode1@btinternet.com
  *  
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,9 +18,17 @@
  *
  */
 
-// make may define: PORTAUDIO, SELFPIPE, RESAMPLE, RESAMPLE_MP, VISEXPORT, DSD, LINKALL to influence build
+// make may define: PORTAUDIO, SELFPIPE, RESAMPLE, RESAMPLE_MP, VISEXPORT, IR, DSD, LINKALL to influence build
 
-#define VERSION "v1.6.4"
+#define VERSION "v1.8"
+
+#if !defined(MODEL_NAME)
+#define MODEL_NAME SqueezeLite
+#endif
+
+#define QUOTE(name) #name
+#define STR(macro)  QUOTE(macro)
+#define MODEL_NAME_STRING STR(MODEL_NAME)
 
 // build detection
 #if defined(linux)
@@ -100,6 +108,13 @@
 #define VISEXPORT 0
 #endif
 
+#if LINUX && defined(IR)
+#undef IR
+#define IR 1
+#else
+#define IR 0
+#endif
+
 #if defined(DSD)
 #undef DSD
 #define DSD       1
@@ -133,6 +148,7 @@
 #define LIBAVCODEC  "libavcodec.so.%d"
 #define LIBAVFORMAT "libavformat.so.%d"
 #define LIBSOXR "libsoxr.so.0"
+#define LIBLIRC "liblirc_client.so.0"
 #endif
 
 #if OSX
@@ -213,6 +229,7 @@
 #define STREAM_THREAD_STACK_SIZE  64 * 1024
 #define DECODE_THREAD_STACK_SIZE 128 * 1024
 #define OUTPUT_THREAD_STACK_SIZE  64 * 1024
+#define IR_THREAD_STACK_SIZE      64 * 1024
 #define thread_t pthread_t;
 #define closesocket(s) close(s)
 #define last_error() errno
@@ -409,7 +426,7 @@ void buf_init(struct buffer *buf, size_t size);
 void buf_destroy(struct buffer *buf);
 
 // slimproto.c
-void slimproto(log_level level, char *server, u8_t mac[6], const char *name, const char *namefile);
+void slimproto(log_level level, char *server, u8_t mac[6], const char *name, const char *namefile, const char *modelname);
 void slimproto_stop(void);
 void wake_controller(void);
 
@@ -440,7 +457,7 @@ void stream_sock(u32_t ip, u16_t port, const char *header, size_t header_len, un
 bool stream_disconnect(void);
 
 // decode.c
-typedef enum { DECODE_STOPPED = 0, DECODE_RUNNING, DECODE_COMPLETE, DECODE_ERROR } decode_state;
+typedef enum { DECODE_STOPPED = 0, DECODE_READY, DECODE_RUNNING, DECODE_COMPLETE, DECODE_ERROR } decode_state;
 
 struct decodestate {
 	decode_state state;
@@ -544,6 +561,7 @@ struct outputstate {
 	u8_t  *track_start;        // set in decode thread
 	u32_t gainL;               // set by slimproto
 	u32_t gainR;               // set by slimproto
+	bool  invert;              // set by slimproto
 	u32_t next_replay_gain;    // set by slimproto
 	unsigned threshold;        // set by slimproto
 	fade_state fade;
@@ -554,6 +572,8 @@ struct outputstate {
 	unsigned fade_secs;        // set by slimproto
 	unsigned rate_delay;
 	bool delay_active;
+	u32_t stop_time;
+	u32_t idle_to;
 #if DSD
 	bool next_dop;             // set in decode thread
 	bool dop;
@@ -562,7 +582,7 @@ struct outputstate {
 #endif
 };
 
-void output_init_common(log_level level, const char *device, unsigned output_buf_size, unsigned rates[]);
+void output_init_common(log_level level, const char *device, unsigned output_buf_size, unsigned rates[], unsigned idle);
 void output_close_common(void);
 void output_flush(void);
 // _* called with mutex locked
@@ -572,17 +592,20 @@ void _checkfade(bool);
 // output_alsa.c
 #if ALSA
 void list_devices(void);
+void list_mixers(const char *output_device);
+void set_volume(unsigned left, unsigned right);
 bool test_open(const char *device, unsigned rates[]);
 void output_init_alsa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[], 
-					  unsigned rate_delay, unsigned rt_priority);
+					  unsigned rate_delay, unsigned rt_priority, unsigned idle, char *volume_mixer, bool mixer_unmute);
 void output_close_alsa(void);
 #endif
 
 // output_pa.c
 #if PORTAUDIO
 void list_devices(void);
+void set_volume(unsigned left, unsigned right);
 bool test_open(const char *device, unsigned rates[]);
-void output_init_pa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[], unsigned rate_delay);
+void output_init_pa(log_level level, const char *device, unsigned output_buf_size, char *params, unsigned rates[], unsigned rate_delay, unsigned idle);
 void output_close_pa(void);
 void _pa_open(void);
 #endif
@@ -611,7 +634,7 @@ void vis_stop(void);
 // dop.c
 #if DSD
 bool is_flac_dop(u32_t *lptr, u32_t *rptr, frames_t frames);
-void update_dop_marker(u32_t *ptr, frames_t frames);
+void update_dop(u32_t *ptr, frames_t frames, bool invert);
 void dop_silence_frames(u32_t *ptr, frames_t frames);
 void dop_init(bool enable, unsigned delay);
 #endif
@@ -627,3 +650,15 @@ struct codec *register_vorbis(void);
 struct codec *register_faad(void);
 struct codec *register_dsd(void);
 struct codec *register_ff(const char *codec);
+
+// ir.c
+#if IR
+struct irstate {
+	mutex_type mutex;
+	u32_t code;
+	u32_t ts;
+};
+
+void ir_init(log_level level, char *lircrc);
+void ir_close(void);
+#endif
